@@ -1,6 +1,7 @@
+import datetime
 from typing import List
 
-from influxdb_client import InfluxDBClient, Point, WriteOptions, WriteApi, QueryApi
+from influxdb_client import InfluxDBClient, Point, WriteOptions, WriteApi, QueryApi, WritePrecision
 from influxdb_client.client.delete_api import DeleteApi
 from influxdb_client.client.flux_table import FluxRecord
 from influxdb_client.client.write_api import WriteType
@@ -15,10 +16,11 @@ class InfluxExperimentDatabase(ExperimentDatabase):
     query: QueryApi
     delete: DeleteApi
 
-    def __init__(self, client: InfluxDBClient, org_name: str = 'galileo') -> None:
+    def __init__(self, client: InfluxDBClient, org_name: str = 'galileo', org_id='org-id') -> None:
         super().__init__()
         self.client = client
         self.org_name = org_name
+        self.org_id = org_id
         self.write_options = WriteOptions(write_type=WriteType.synchronous)
 
     def open(self):
@@ -41,28 +43,6 @@ class InfluxExperimentDatabase(ExperimentDatabase):
     def get_experiment(self, exp_id: str) -> Experiment:
         raise NotImplementedError()
 
-    def save_traces(self, traces: List[RequestTrace]):
-        if len(traces) == 0:
-            return
-
-        points: List[Point] = list()
-        for trace in traces:
-            p = Point("trace") \
-                .time(int(trace.created)) \
-                .field("request_id", trace.request_id) \
-                .tag("client", trace.client) \
-                .tag("service", trace.service) \
-                .tag("created", trace.created) \
-                .tag("sent", trace.sent) \
-                .tag("done", trace.done) \
-                .tag("status", trace.status) \
-                .tag("server", trace.server) \
-                .tag("exp_id", trace.exp_id) \
-                .tag("response", trace.response)
-            points.append(p)
-
-        self.writer.write(bucket=traces[0].exp_id, org=self.org_name, record=points)
-
     def touch_traces(self, experiment: Experiment):
         raise NotImplementedError()
 
@@ -78,19 +58,12 @@ class InfluxExperimentDatabase(ExperimentDatabase):
             status=int(record.values['status']),
             server=record.values['server'],
             exp_id=record.values['exp_id'],
+            headers=record.values.get('headers', None),
             response=record.values['response']
         )
 
     def get_traces(self, exp_id: str) -> List[RequestTrace]:
-        records = self.query.query_stream(
-            f'''
-            from(bucket:"{exp_id}")
-              |> range(start: 1970-01-01)
-              |> filter(fn: (r) =>
-                  r._measurement == "trace"
-              )
-            ''', org=self.org_name
-        )
+        records = self._query_for_measurement("traces", exp_id)
         events = list()
 
         for record in records:
@@ -100,33 +73,8 @@ class InfluxExperimentDatabase(ExperimentDatabase):
 
     # https://github.com/influxdata/influxdb-client-python/blob/eadbf6ac014582127e2df54698682e2924973e19/examples/nanosecond_precision.py#L37
 
-    def save_telemetry(self, telemetry: List[Telemetry]):
-        if len(telemetry) == 0:
-            return
-
-        points: List[Point] = list()
-        for data in telemetry:
-            p = Point("telemetry") \
-                .time(int(data.timestamp)) \
-                .tag('ts', data.timestamp) \
-                .field('value', data.value) \
-                .tag('exp_id', data.exp_id) \
-                .tag('node', data.node) \
-                .tag('metric', data.metric)
-            points.append(p)
-
-        self.writer.write(bucket=telemetry[0].exp_id, org=self.org_name, record=points)
-
     def get_telemetry(self, exp_id=None) -> List[Telemetry]:
-        records = self.query.query_stream(
-            f'''
-            from(bucket:"{exp_id}")
-              |> range(start: 1970-01-01)
-              |> filter(fn: (r) =>
-                  r._measurement == "telemetry"
-              )
-            ''', org=self.org_name
-        )
+        records = self._query_for_measurement("telemetry", exp_id)
         events = list()
 
         for record in records:
@@ -141,12 +89,56 @@ class InfluxExperimentDatabase(ExperimentDatabase):
             metric=record.values['metric'],
             node=record.values['node'],
             value=record.get_value(),
-            exp_id=record.values['exp_id']
+            exp_id=record.values['exp_id'],
+            subsystem=record.values.get('subsystem')
         )
 
+    def save_telemetry(self, telemetry: List[Telemetry]):
+        if len(telemetry) == 0:
+            return
+
+        points: List[Point] = list()
+        for data in telemetry:
+            strftime = datetime.datetime.utcfromtimestamp(float(data.timestamp))
+            p = Point("telemetry") \
+                .time(strftime, WritePrecision.MS) \
+                .tag('ts', float(data.timestamp)) \
+                .field('value', float(data.value)) \
+                .tag('exp_id', data.exp_id) \
+                .tag('node', data.node) \
+                .tag('metric', data.metric) \
+                .tag('subsystem', data.subsystem)
+            points.append(p)
+
+        self.writer.write(bucket=telemetry[0].exp_id, org=self.org_name, record=points)
+
     def save_event(self, event: ExperimentEvent):
         return self.save_events([event])
 
+    def save_traces(self, traces: List[RequestTrace]):
+        if len(traces) == 0:
+            return
+
+        points: List[Point] = list()
+        for trace in traces:
+            strftime = datetime.datetime.utcfromtimestamp(float(trace.sent))
+            p = Point("traces") \
+                .time(strftime, WritePrecision.MS) \
+                .field("request_id", trace.request_id) \
+                .tag("client", trace.client) \
+                .tag("service", trace.service) \
+                .tag("created", float(trace.created)) \
+                .tag("sent", float(trace.sent)) \
+                .tag("done", float(trace.done)) \
+                .tag("status", int(trace.status)) \
+                .tag("server", trace.server) \
+                .tag("exp_id", trace.exp_id) \
+                .tag("response", trace.response) \
+                .tag("headers", trace.headers)
+            points.append(p)
+
+        self.writer.write(bucket=traces[0].exp_id, org=self.org_name, record=points)
+
     def save_events(self, events: List[ExperimentEvent]):
         if len(events) == 0:
             return
@@ -154,9 +146,10 @@ class InfluxExperimentDatabase(ExperimentDatabase):
         points: List[Point] = list()
 
         for event in events:
-            p = Point("event") \
-                .time(int(event.timestamp)) \
-                .tag("ts", event.timestamp) \
+            strftime = datetime.datetime.utcfromtimestamp(float(event.timestamp))
+            p = Point("events") \
+                .time(strftime, WritePrecision.MS) \
+                .tag("ts", float(event.timestamp)) \
                 .tag("name", event.name) \
                 .field("value", event.value) \
                 .tag("exp_id", event.exp_id)
@@ -168,32 +161,32 @@ class InfluxExperimentDatabase(ExperimentDatabase):
     def _map_flux_record_to_exp_event(record: FluxRecord) -> ExperimentEvent:
         return ExperimentEvent(
             exp_id=record.values['exp_id'],
-            timestamp=record.values['ts'],
+            timestamp=float(record.values['ts']),
             name=record.values['name'],
             value=record.get_value()
         )
 
-    def get_events(self, exp_id) -> List[ExperimentEvent]:
-        tables = self.query.query(
-            f'''
-            from(bucket:"{exp_id}")
-              |> range(start: 1970-01-01)
-              |> filter(fn: (r) =>
-                  r._measurement == "event"
-              )
+    def _query_for_measurement(self, measurement: str, exp_id: str):
+        stop = datetime.datetime.utcnow() + datetime.timedelta(days=1)
+        stop = stop.strftime('%Y-%m-%dT%H:%M:%S.%fZ')
+        records = self.query.query_stream(
+            f'''  
+               from(bucket: "{exp_id}")
+                 |> range(start: 1970-01-01, stop: {stop})
+                 |> filter(fn: (r) => r["_measurement"] == "{measurement}")    
             '''
         )
-        if len(tables) == 0:
-            return []
-
-        records = tables[0].records
-        events = list()
+        return records
 
+    def get_events(self, exp_id) -> List[ExperimentEvent]:
+        records = self._query_for_measurement("events", exp_id)
+        events = []
         for record in records:
             events.append(InfluxExperimentDatabase._map_flux_record_to_exp_event(record))
 
         return events
 
+
     def save_nodeinfos(self, infos: List[NodeInfo]):
         raise NotImplementedError()
 
